/*
   FNT - Stream handler for socket connections

   Copyright (C) 2009 Free Software Foundation, Inc.

   Written by:
   Slava Zanko <slavazanko@gmail.com>


   This file is part of the FIDO New Technology.

   FIDO New Technology is free software: you can redistribute it and/or 
   modify it under the terms of the GNU General Public License as published 
   by the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <string.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>

#include "../include/global.h"
#include "../include/types.h"
#include "../include/error.h"
#include "../include/plugin.h"
#include "../include/stream.h"
#include "../include/mainloop.h"

#include "main.h"
#include "utils.h"

/*** global variables **************************************************/

/*** file scope macro definitions **************************************/

/*** file scope type declarations **************************************/

/*** file scope variables **********************************************/

#ifdef G_OS_WIN32
static WSADATA wsaData = NULL;
#endif

/*** file scope functions **********************************************/

/*** public functions **************************************************/

DLLEXPORT fnt_plugin_info_t *
fnt_plugin_get_info (void)
{
    fnt_plugin_info_t *info;

    info = g_try_malloc (sizeof (fnt_plugin_info_t));
    if (info == NULL)
	return NULL;

    info->magic = FNT_PLUGIN_MAGIC;
    info->group = "stream";
    info->name = "socket";
    return info;
}


DLLEXPORT bool
init (void)
{
#ifdef G_OS_WIN32
    WSADATA wsaData;
    if (WSAStartup (MAKEWORD (2, 2), &wsaData) == SOCKET_ERROR)
	return false;
#endif
    return true;
}

DLLEXPORT bool
deinit (void)
{
    return true;
}

DLLEXPORT bool
assignToMainloop (fnt_stream_t * stream, fnt_mainloop_t * mainloop,
		  GIOFunc callbackIo)
{
    __fnt_driver_data_t *driver_data;

    if (stream->driver_data == NULL)
	return false;
    driver_data = (__fnt_driver_data_t *) stream->driver_data;

    stream->mainloop = mainloop;

    if (stream->is_server)
    {
	driver_data->callbackIo = callbackIo;
	stream->g_event_source = g_io_add_watch (driver_data->socket, G_IO_IN
						  /*| G_IO_PRI | G_IO_ERR | G_IO_HUP */
						  ,
						  fnt_stream_socket_acceptCallback,
						  (gpointer) stream);
    }
    else
    {
	stream->g_event_source =
	    g_io_add_watch (driver_data->socket,
			    G_IO_IN | G_IO_PRI | G_IO_ERR | G_IO_HUP,
			    callbackIo, stream);
    }
    g_io_channel_unref (driver_data->socket);
    return true;
}
